import JenkinsConfig
import jenkins
import sys
import xml.etree.ElementTree as ET


class JenkinsJobException(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)


class JenkinsJob:
    """Provides access to common Jenkins job attributes."""

    STATUS_UNKNOWN = "Unknown"
    STATUS_PENDING = "Pending"
    STATUS_ACTIVE = "Active"
    STATUS_COMPLETE = "Complete"

    def __init__(self, server, job):
        self.server = server
        self.job = job
        self.name = job['name']
        self.info = server.get_job_info(job['name'])
        self.xml = server.get_job_config(job['name'])
        self.xmltree = ET.ElementTree(ET.XML(self.xml))
        #print("XML: %s" % (self.xml))
        self.status = JenkinsJob.STATUS_UNKNOWN
        if "anime" in self.info['color']:
            self.status = JenkinsJob.STATUS_ACTIVE
        elif self.info['inQueue']:
            self.status = JenkinsJob.STATUS_PENDING
        else:
            self.status = JenkinsJob.STATUS_COMPLETE

    def __str__(self):
        return repr(self.info)

    def findXMLSubElements(self, tree, match=None):
        """Returns all child elements of tree matching match."""
        element_list = []
        #tree = ET.ElementTree(element)
        item = tree.iter(match)
        for i in item:
            #print("i: %s" % (i))
            element_list.append(i)

        return element_list

    def iterMethod(self, item_to_find, tag):
        """Returns the XML element for the trigger scripts, for testing"""
        element = ET.XML(self.xml)
        tree = ET.ElementTree(element)
        item = tree.iter(item_to_find)
        for i in item:
            #print("i: %s" % (i))
            for sub_element in list(i.iter()):
                #print("sub_element: %s" % (sub_element))
                if sub_element.tag == tag:
                    #print("tag: %s" % sub_element.tag)
                    #print("text: %s" % sub_element.text)
                    #print("tail: %s" % sub_element.tail)
                    #print("attrib: %s" % sub_element.attrib)
                    # TODO this only assumes 1 script trigger
                    return sub_element

        return None

    def getParameterList(self):
        """Extracts the parameter list as a list of elements"""
        element_list = self.findXMLSubElements(self.xmltree,
                                               "parameterDefinitions")
        if len(element_list) != 1:
            raise JenkinsJobException("Unexpected parameterDefinitions count")
        name_list = self.findXMLSubElements(element_list[0], "name")
        if len(name_list) == 0:
            raise JenkinsJobException("Unexpected name count")
        self.parameter_list = name_list

    def getPostBuildProjects(self):
        """Extracts the post build trigger projects as a single element"""
        tag = "hudson.plugins.parameterizedtrigger.BuildTriggerConfig"
        element_list = self.findXMLSubElements(self.xmltree, tag)
        if len(element_list) != 1:
            raise JenkinsJobException("Unexpected %s count" % (tag))
        tag = "projects"
        project_list = self.findXMLSubElements(element_list[0], tag)
        if len(element_list) != 1:
            raise JenkinsJobException("Unexpected %s count" % (tag))
        self.post_build_projects = project_list[0]

    def isRunning(self):
        if self.status in [JenkinsJob.STATUS_ACTIVE,
                           JenkinsJob.STATUS_PENDING]:
            return True

        return False


def processJobs(j):
    job_list = j.get_jobs()

    for raw_job in job_list:
        if raw_job["name"].startswith("template"):
            job = JenkinsJob(j, raw_job)
            job.getParameterList()
            print("%s: %s" % (job.name, job.status))


def main():
    config = JenkinsConfig.JenkinsConfig(sys.argv[1])
    j = jenkins.Jenkins(config.hostname,
                        username=config.username,
                        password=config.password)
    processJobs(j)


if __name__ == "__main__":
    main()
