import JenkinsConfig
import jenkins
import sys
import xml.etree.ElementTree as ET


class JenkinsJobException(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)


class JenkinsJob:
    """Provides access to the Jenkins Job XML."""

    def __init__(self, xml):
        #self.xml = xml
        self.xmltree = ET.ElementTree(ET.XML(xml))
        #print("XML: %s" % (self.xml))

        self.parameter_list = None
        #self.postBuildProject = None
        self.postBuildProjects = []

        self.getParameterList()
        self.getPostBuildProjects()

    def __str__(self):
        return repr(self.xmltree)

    def write(self, file_name):
        self.xmltree.write(file_name)

    def findXMLSubElements(self, tree, match=None):
        """Returns all child elements of tree matching match."""
        element_list = []
        #tree = ET.ElementTree(element)
        item = tree.iter(match)
        for i in item:
            #print("i: %s" % (i))
            element_list.append(i)

        return element_list

    def findXMLElementList(self, tag_list, root=None):
        """Traverses a list of xml tags in order and returns the list of
        matching xml elements.

        All but the last tag must have only one child element."""
        if root is None:
            root = self.xmltree
        for tag in tag_list:
            sub = self.findXMLSubElements(root, tag)
            if not len(sub) == 1:
                raise JenkinsJobException("Unexpected %s count" % (tag))
            root = sub[0]
        return root

    def getParameterList(self):
        """Extracts the parameter list as a list of elements"""
        tag = "parameterDefinitions"
        element_list = self.findXMLSubElements(self.xmltree, tag)
        if len(element_list) != 1:
            raise JenkinsJobException("Unexpected %s count" % (tag))
        tag = "hudson.model.StringParameterDefinition"
        name_list = self.findXMLSubElements(element_list[0], tag)
        if len(name_list) == 0:
            raise JenkinsJobException("Unexpected %s count" % (tag))
        self.parameter_list = name_list

    def getPostBuildProjects(self):
        """Extracts the post build trigger projects as a single element"""
        tag = "hudson.plugins.parameterizedtrigger.BuildTriggerConfig"
        postBuildTriggerConfig = self.findXMLSubElements(self.xmltree, tag)
        #if len(element_list) != 1:
        #    raise JenkinsJobException("Unexpected %s count" % (tag))
        tag = "projects"
        for element in postBuildTriggerConfig:
            #print("getPostBuildProjects [element]:")
            #self.dumpElement(element)
            projects = self.findXMLSubElements(element, tag)
            #if len(projects) != 1:
            #    raise JenkinsJobException("Unexpected %s count" % (tag))
            for p in projects:
                #print("getPostBuildProjects [p]:")
                #self.dumpElement(p)
                # TODO: Find a better way to match the post-build project
                if "template" in p.text:
                    self.postBuildProjects.append(p)

                    #print("Found it!")
                    # Assume that this is the correct post-build project
                    #if self.postBuildProject == None:
                    #    self.postBuildProject = p
                    #else:
                    #    raise JenkinsJobException("Unexpected Post-build " +
                    #                              "template count")

    def setParameter(self, in_name, in_value):
        #print("setParameter: %s = %s" % (in_name, in_value))
        for p in self.parameter_list:
            name_list = self.findXMLSubElements(p, "name")
            # TODO: add error checking
            name = name_list[0]
            #print("p name: %s" % (name.text))
            if name.text == in_name:
                defaultValue_list = self.findXMLSubElements(p, "defaultValue")
                defaultValue = defaultValue_list[0]
                #print("p value: %s" % (defaultValue.text))
                defaultValue.text = in_value
        #print("XML Parameters: %s" % (self.parameter_list))

    def toString(self):
        return ET.tostring(self.xmltree.getroot())

    def dumpElement(self, element):
        print("tag: %s" % element.tag)
        print("text: %s" % element.text)
        print("tail: %s" % element.tail)
        print("attrib: %s" % element.attrib)

    def setPostBuildProjects(self, projects):
        #print("Before:")
        #self.dumpElement(self.postBuildProject)
        if len(self.postBuildProjects) < 1:
            raise JenkinsJobException("Not defined")
        for p in self.postBuildProjects:
            p.text = projects
        #print("After:")
        #self.dumpElement(self.postBuildProject)


def processJobs(j):
    job_list = j.get_jobs()

    for raw_job in job_list:
        if raw_job["name"].startswith("template"):
            job = JenkinsJob(j, raw_job)
            job.getParameterList()
            print("%s: %s" % (job.name, job.status))


def getJob(j, job_name):
    job = j.get_job_config(job_name)
    jjxml = JenkinsJob(job)
    jjxml.write("%s.xml" % job_name)

    element_path = ["matrix-project",
                    "triggers",
                    "org.jenkinsci.plugins.scripttrigger.ScriptTrigger",
                    "script"]
    element = jjxml.findXMLElementList(element_path)
    print("Found: %s" % (element))
    script = element.text
    script = script.replace("$target_branch", "FOO_BAR")
    element.text = script
    jjxml.write("%s-modified.xml" % job_name)


def main():
    config = JenkinsConfig.JenkinsConfig(sys.argv[1])
    j = jenkins.Jenkins(config.hostname,
                        username=config.username,
                        password=config.password)
    getJob(j, sys.argv[2])
    #processJobs(j)


if __name__ == "__main__":
    main()
