#!/usr/bin/env python

import jenkins
import logging
import networkx as nx
import sys

import JenkinsVisitor as jv
import JenkinsConfig
import JenkinsJobXML
import ProjectConfig
from PackageDependencies import PackageDependencies
import project as p

DOT_FILE = "dep.dot"
PICKLE_FILE = "mbs_dependency_graph.nx"


class MetaBuildSystemException(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)


class MetaBuildSystem:
    def __init__(self, project_config, jenkins_config):
        self.project_config = project_config
        self.jenkins_config = jenkins_config
        self.jenkins_server = jenkins.Jenkins(jenkins_config.hostname,
                                              jenkins_config.username,
                                              jenkins_config.password)

        # Populate the root
        self.root_project = p.Project(project_config.getRoot(), -1)
        project_config.reviseProjectInfo(self.root_project)

        # Generate the graph with the root project
        self.dependency_graph = nx.DiGraph()

        # Other stuff
        self.local_templates = False
        self.preview = True

    def buildGraph(self, package_repo):
        """Builds the project dependency graph"""
        depBuilder = PackageDependencies(package_repo, self.dependency_graph)
        project_list = []
        for proj in self.project_config.table:
            project = p.Project(proj, -1)
            self.project_config.reviseProjectInfo(project)
            project_list.append(project)

        for project in project_list:
            depBuilder.generateDeps(project)
        update = True
        while update:
            for project in project_list:
                update = depBuilder.add_to_dependency_graph(project)

        self.dump_graph()
        self.executeMapping(depBuilder.recordProjectDependencies)
        self.dump_graph()

    def dump_graph(self):
        log_str = "Node count: %s" % (self.dependency_graph.number_of_nodes())
        logging.getLogger(__name__).info(log_str)

    def executeMapping(self, func):
        """Applies the given function to a top-down sorting of the graph"""
        if (not nx.is_directed_acyclic_graph(self.dependency_graph)):
            logging.getLogger(__name__).error("Not a DAG")
            print("Not a DAG")
            return False
        schedule = nx.topological_sort(self.dependency_graph)
        map(func, schedule)
        return True

    def genJobName(self, base, series, suffix):
        """Assembles the naming components into a jenkins job name"""
        if series == "":
            return "%s-%s" % (base, suffix)
        return "%s-%s-%s" % (base, series, suffix)

    def getJob(self, template):
        """Retrieves the template job from the jenkins server"""
        if self.local_templates:
            with open("%s.xml" % (template)) as f:
                job = f.read()
        else:
            job = self.jenkins_server.get_job_config(template)
        return JenkinsJobXML.JenkinsJob(job)

    def setJobParameters(self, jjxml, project):
        """Set jenkins job parameters"""
        jjxml.setParameter("project", project.name)
        for p in project.parameters:
            #print("Project Parameter: %s" % (p))
            jjxml.setParameter(p, project.parameters[p])

    def scriptTriggerWorkaround(self, jjxml, project):
        """Modifies the jenkins job script trigger text to replace the
        target_branch variable"""
        element_path = ["matrix-project",
                        "triggers",
                        "org.jenkinsci.plugins.scripttrigger.ScriptTrigger",
                        "script"]
        script_trigger = jjxml.findXMLElementList(element_path)
        # Jenkins does not replace variables in the polling trigger.
        # Do it manually.
        text = script_trigger.text.replace("$target_branch",
                                           project.parameters["target_branch"])
        script_trigger.text = text

    def publishJob(self, name, jjxml):
        """Publishes the jenkins job to the jenkins server"""
        if not self.preview:
            if self.jenkins_server.job_exists(name):
                self.jenkins_server.reconfig_job(name, jjxml.toString())
            else:
                self.jenkins_server.create_job(name, jjxml.toString())
        jjxml.write("%s.xml" % (name))

    def createCiJob(self, project, template, ci_suffix):
        """Clones the ci template and updates with the job specific details"""
        name = self.genJobName(project.name,
                               self.project_config.series,
                               ci_suffix)

        jjxml = self.getJob(template)
        self.setJobParameters(jjxml, project)
        self.scriptTriggerWorkaround(jjxml, project)
        self.publishJob(name, jjxml)

    def createAlJob(self, project, template, al_suffix, dput_suffix):
        """Clones the al template and updates with the job specific details"""
        name = self.genJobName(project.name,
                               self.project_config.series,
                               al_suffix)
        # The al script will trigger the same project check-pt script
        triggerName = self.genJobName(project.name,
                                      self.project_config.series,
                                      dput_suffix)

        jjxml = self.getJob(template)
        jjxml.setPostBuildProjects(triggerName)
        jjxml.setParameter("postbuild_projects", triggerName)
        self.setJobParameters(jjxml, project)
        self.scriptTriggerWorkaround(jjxml, project)
        self.publishJob(name, jjxml)

    def createCheckPtJob(self, project, template, check_suffix, build_suffix):
        """Clones the pt check template and updates with the job specific
        details"""
        name = self.genJobName(project.name,
                               self.project_config.series,
                               check_suffix)
        # The check script will trigger the same project build script
        triggerName = self.genJobName(project.name,
                                      self.project_config.series,
                                      build_suffix)

        jjxml = self.getJob(template)
        # The check script will abort if any upstream branches have approved
        # merge requests. Only the name of the project target branch is needed.
        #print("upStream: %s" % (project.upStream))
        jjxml.setParameter("prebuild_branches", " ".join(project.upStream))
        jjxml.setPostBuildProjects(triggerName)
        jjxml.setParameter("postbuild_projects", triggerName)
        self.setJobParameters(jjxml, project)
        self.publishJob(name, jjxml)

    def createBuildPtJob(self, project, template, build_suffix, dput_suffix):
        """Clones the pt build template and updates with the job specific
        details"""
        name = self.genJobName(project.name,
                               self.project_config.series,
                               build_suffix)

        # The build script will trigger the same project dput script
        triggerName = self.genJobName(project.name,
                                      self.project_config.series,
                                      dput_suffix)

        jjxml = self.getJob(template)
        jjxml.setPostBuildProjects(triggerName)
        jjxml.setParameter("postbuild_projects", triggerName)
        self.setJobParameters(jjxml, project)
        self.publishJob(name, jjxml)

    def createDputPtJob(self, project, template, dput_suffix, check_suffix):
        """Clones the pt dput template and updates with the job specific
        details"""
        name = self.genJobName(project.name,
                               self.project_config.series,
                               dput_suffix)

        # The build script will trigger downstream check scripts
        # Generate the expected jenkins job names from the project names
        #print("downStream: %s" % (project.downStream))
        post_build_list = []
        for p in project.downStream:
            triggerName = self.genJobName(p,
                                          self.project_config.series,
                                          check_suffix)
            post_build_list.append(triggerName)

        jjxml = self.getJob(template)
        jjxml.setPostBuildProjects(", ".join(post_build_list))
        jjxml.setParameter("postbuild_projects", ", ".join(post_build_list))
        self.setJobParameters(jjxml, project)
        self.publishJob(name, jjxml)

    def createCiJobs(self, project):
        """Creates a jenkins ci job from a project"""
        ci_suffix = self.jenkins_config.ci_suffix
        template = self.jenkins_config.ci_template
        self.createCiJob(project, template, ci_suffix)

    def createAlJobs(self, project):
        """Creates a jenkins al job from a project"""
        al_suffix = self.jenkins_config.al_suffix
        dput_suffix = self.jenkins_config.pt_dput_suffix
        template = self.jenkins_config.al_template
        self.createAlJob(project, template, al_suffix, dput_suffix)

    def createPtJobs(self, project):
        """Creates the jenkins pt jobs from a project"""
        check_suffix = self.jenkins_config.pt_check_suffix
        build_suffix = self.jenkins_config.pt_build_suffix
        dput_suffix = self.jenkins_config.pt_dput_suffix
        template = self.jenkins_config.pt_check_template
        self.createCheckPtJob(project, template, check_suffix, build_suffix)
        template = self.jenkins_config.pt_build_template
        self.createBuildPtJob(project, template, build_suffix, dput_suffix)
        template = self.jenkins_config.pt_dput_template
        self.createDputPtJob(project, template, dput_suffix, check_suffix)

    def createJobs(self):
        """Create Jenkins jobs from the dependency tree"""
        self.build_queue = []
        jenkinsVisitor = jv.JenkinsVisitor(self.dependency_graph,
                                           self.build_queue)
        self.executeMapping(jenkinsVisitor.map_create)
        for proj in self.build_queue:
            # TODO: Just do select jobs
            print("project: %s" % (proj))
            #if not proj.name in ["dee"]:
            #    print(" - skipped")
            #    continue

            # TODO: Skip creation of jobs
            #continue

            # Clone the template job and update the project specific details
            self.createCiJobs(proj)
            self.createAlJobs(proj)
            self.createPtJobs(proj)

    def save(self, pickle_file, dot_file):
        """Save the dependency graph, mostly for analysis through graphviz"""
        self.dump_graph()
        logging.getLogger(__name__).info("Saving gpickle format: %s" %
                                         pickle_file)
        nx.write_gpickle(self.dependency_graph, pickle_file)
        logging.getLogger(__name__).info("Saving dot format: %s" % dot_file)
        nx.write_dot(self.dependency_graph, dot_file)
        self.dump_graph()

"""
def main():
    # TODO: replace with arg_parser
    project_config_file_name = sys.argv[1]
    jenkins_config_file_name = sys.argv[2]
    try:
        project_config = ProjectConfig.ProjectConfig(project_config_file_name)
        logging.getLogger(__name__).info("Project Config: %s" %
                                         project_config)
        apt_pkg.init()
        apt_pkg.DepCache(apt_pkg.Cache())
        project_config.initializeSources(apt_pkg)
        jenkins_config = JenkinsConfig.JenkinsConfig(jenkins_config_file_name)
        logging.getLogger(__name__).info("Jenkins Config: %s" %
                                         jenkins_config)
        mbs = MetaBuildSystem(project_config, jenkins_config)
        mbs.buildGraph(apt_pkg)
        mbs.save(PICKLE_FILE, DOT_FILE)
        mbs.createJobs()
    except KeyError, err:
        logging.getLogger(__name__).error("ERROR: %s" % (err))
        print("%s" % (err))
        return -1

    return 0
"""


if __name__ == "__main__":
    FORMAT = '%(asctime)-15s %(name)s %(message)s'
    logging.basicConfig(filename='MbsNew.log', filemode='w', format=FORMAT)
    logging.getLogger().setLevel(logging.INFO)
    sys.exit(main())
