import ConfigParser
import logging
import sys

import project as prj

ROOT_SECTION = "root"
GLOBAL_SECTION = "global"
REQUIRED_OPTIONS = ["pbuilder_hooks",
                    "target_branch",
                    "packaging_branch",
                    "local_archive",
                    "local_archive_host",
                    "local_archive_login",
                    "local_archive_tmp",
                    "local_archive_source",
                    "local_archive_pocket",
                    "ppa",
                   ]
OPTIONAL_OPTIONS = ["requires",
                    "provides",
                   ]


class ProjectConfig():
    """Manages identification of the PPA dependencies within a set of
    interdependent projects."""

    def __init__(self, file_name):
        self.file_name = file_name
        self.table = {}
        self.root = None
        self.series = ""

        self.config = ConfigParser.SafeConfigParser()
        self.config.read([file_name])

        sections = self.config.sections()

        if ROOT_SECTION not in sections:
            raise KeyError("%s: Missing [%s] section" %
                           (self.file_name, ROOT_SECTION))
        if GLOBAL_SECTION not in sections:
            raise KeyError("%s: Missing [%s] section" %
                           (self.file_name, GLOBAL_SECTION))

        for section in sections:
            if section == ROOT_SECTION:
                # Special handling
                if not self.config.has_option(section, "root"):
                    raise KeyError("%s: Missing option 'root', in [%s]" %
                                   (self.file_name, section))
                self.root = self.config.get(section, "root")
                if self.config.has_option(section, "series"):
                    self.series = self.config.get(section, "series")
                continue

            if section == GLOBAL_SECTION:
                continue

            self.table[section] = {}

            # Add the global options to each project section
            for option in self.config.options(GLOBAL_SECTION):
                self.table[section][option] = self.config.get(GLOBAL_SECTION,
                                                              option)
            # Build the per project section table
            # This allows project specific options to override the globals
            for option in self.config.options(section):
                self.table[section][option] = self.config.get(section, option)

            # Check for missing options
            for option in REQUIRED_OPTIONS:
                if not option in self.table[section]:
                    raise KeyError("%s: Missing option '%s', for [%s]" %
                                   (self.file_name, option, section))

    def initializeSources(self, package_repo):
        for section in self.config.sections():
            if section in [ROOT_SECTION, GLOBAL_SECTION]:
                continue
            # Extract any provides data from the config file,
            # then attempt to acquire better data from the apt source data.
            provides = self.table[section].get("provides")
            if not provides is None:
                provides = provides.split()
            log_str = ("initializeSources: %s -> config (%s)" %
                       (section, provides))
            logging.getLogger(__name__).info(log_str)
            provides = self.getBinariesFromSource(section, provides,
                                                  package_repo)
            log_str = ("initializeSources: %s -> apt (%s)" %
                       (section, provides))
            logging.getLogger(__name__).info(log_str)
            if (provides is None) or (len(provides) == 0):
                # A source project was not found and has no manually specified
                # provided packages, abort.
                raise KeyError("%s: No binary packages found for [%s]" %
                               (self.file_name, section))
            self.table[section]["provides"] = provides

    def __str__(self):
        """Returns a readable string version"""
        return repr("[root: '%s'] -> %s" % (self.root, self.table))

    def getBinariesFromSource(self, source, provides, package_repo):
        """Identifies the binary packages from a source package, if none found
        just return provides."""
        binaries = package_repo.get_binaries(source)
        if len(binaries) > 0:
            return binaries
        return provides

    def getRoot(self):
        """Returns the root section"""
        return self.root

    def reviseProjectInfo(self, project):
        """Populates a project node from data supplied by the project config"""
        logging.getLogger(__name__).info("reviseProjectInfo: %s" % (project))
        if project.name not in self.table:
            return False

        section = self.table[project.name]
        for param in section:
            project.parameters[param] = section[param]
        project.manualDependencies = []
        try:
            # The following keys are optional
            for req in section['requires'].split():
                project.manualDependencies.append(req)
            logging.getLogger(__name__).info("manualDependencies: %s" %
                                             (project.manualDependencies))
        except KeyError:
            pass

        project.packages = set(section['provides'])
        project.isSource = True
        project.graph_node = None
        return True
