from mock import patch
import unittest
import logging
from AptPackageRepository import *


class SourceRecordsFake():
    """Implements very limited apt_pkg-like functionality for testing"""
    def __init__(self, fake_sources):
        self.sources = fake_sources

    def lookup(self, package):
        if package in self.sources:
            self.build_depends = self.sources[package]["build_depends"]
            self.binaries = self.sources[package]["binaries"]
            return True
        return False

    def __call__(self):
        return self


class AptPackageRepositoryTestCase(unittest.TestCase):
    """Unit tests for class ProjectConfig"""

    generic_build_deps_set = set(["build_dep-1", "build_dep-2", "build_dep-3"])
    generic_build_deps = {"Build-Depends": [(("build_dep-1", ), ),
                                            (("build_dep-2", ), ),
                                            (("build_dep-3", ), ), ]}
    generic_binaries = ["binary-1", "binary-2", "binary-3"]

    def setUp(self):
        """Setup for all test cases"""
        self.repo = AptPackageRepository()
        self._initialize_fake(self.repo)

    def tearDown(self):
        """Tear down for all test cases"""
        pass

    def _initialize_fake(self, repo):
        if repo.ready:
            return
        repo.ready = True

    def test_get_build_dependencies(self):
        """Verify that the correct dependences are returned for a package
        name in the repository"""
        record = {"compiz": {"build_depends": self.generic_build_deps,
                             "binaries": self.generic_binaries}}
        source_records_fake = SourceRecordsFake(record)
        with patch("apt_pkg.SourceRecords", source_records_fake):
            actual = self.repo.get_build_dependencies('compiz')
            self.assertEqual(actual, self.generic_build_deps_set)

    def test_get_build_dependencies_none(self):
        """Verify that the empty set is returned for a package name not in
        the repository"""
        record = {"compiz": {"build_depends": self.generic_build_deps,
                             "binaries": self.generic_binaries}}
        source_records_fake = SourceRecordsFake(record)
        with patch("apt_pkg.SourceRecords", source_records_fake):
            actual = self.repo.get_build_dependencies('nux')
            self.assertEqual(actual, set())

    def test_get_binaries(self):
        """Verify that the correct binaries are returned for a package
        name in the repository"""
        record = {"compiz": {"build_depends": self.generic_build_deps,
                             "binaries": self.generic_binaries}}
        source_records_fake = SourceRecordsFake(record)
        with patch("apt_pkg.SourceRecords", source_records_fake):
            actual = self.repo.get_binaries('compiz')
            self.assertEqual(actual, set(self.generic_binaries))

    def test_get_binaries_none(self):
        """Verify that the empty set is returned for a package name not in
        the repository"""
        record = {"compiz": {"build_depends": self.generic_build_deps,
                             "binaries": self.generic_binaries}}
        source_records_fake = SourceRecordsFake(record)
        with patch("apt_pkg.SourceRecords", source_records_fake):
            actual = self.repo.get_binaries('nux')
            self.assertEqual(actual, set())
