import networkx
import unittest
import os.path
import project
from PackageDependencies import *
from PackageRepositoryFake import PackageRepositoryFake

DATA_PATH = "tests/data"
PROJECT_CONFIG_NAME = "project_tests.cfg"


class PackageDependenciesTestCase(unittest.TestCase):
    """Unit tests for class ProjectConfig"""
    generic_packages = {"package-1": "package-1",
                        "package-2": "package-2",
                        "package-3": "package-3"}
    generic_build_depends = ['build_dep-1', 'build_dep-2', 'build_dep-3']
    generic_repo = {"compiz": {"build_dependencies": ["compiz_dep-1",
                                                      "compiz_dep-2",
                                                      "compiz_dep-3",
                                                     ],
                               "binaries": ["compiz_bin-1",
                                            "compiz_bin-2",
                                            "compiz_bin-3",
                                           ],
                              },
                    "nux": {"build_dependencies": ["nux_dep-1",
                                                   "nux_dep-2",
                                                   "nux_dep-3",
                                                  ],
                            "binaries": ["nux_bin-1",
                                         "nux_bin-2",
                                         "nux_bin-3",
                                        ],
                           },
                    "unity": {"build_dependencies": ["unity_dep-1",
                                                     "unity_dep-2",
                                                     "unity_dep-3",
                                                     "compiz_bin-2",
                                                     "nux_bin-1",
                                                    ],
                              "binaries": ["unity_bin-1",
                                           "unity_bin-2",
                                           "unity_bin-3",
                                          ],
                             },
                   }

    def setUp(self):
        """Setup for all test cases"""
        pass

    def tearDown(self):
        """Tear down for all test cases"""
        pass

    def test_GenerateDeps(self):
        """Verify that dependencies are found for a project"""
        fake_repo = PackageRepositoryFake()
        #fake_repo.build_depends = set(self.generic_build_depends)
        fake_repo.repo = self.generic_repo
        graph = networkx.DiGraph()
        pd = PackageDependencies(fake_repo, graph)
        p = project.Project("compiz", -1)
        p.packages = set(["compiz", "compiz-dev"])
        p.manualDependencies = []
        pd.generateDeps(p)
        expected = set(self.generic_repo["compiz"]["build_dependencies"])
        self.assertEqual(expected, p.build_dependencies)

    def test_GenerateDeps_with_manual_dependencies(self):
        """Verify that dependencies are found for a project and manual
        dependencies are added"""
        fake_repo = PackageRepositoryFake()
        #fake_repo.build_depends = set(self.generic_build_depends)
        fake_repo.repo = self.generic_repo
        graph = networkx.DiGraph()
        pd = PackageDependencies(fake_repo, graph)
        p = project.Project("compiz", -1)
        p.packages = set(["compiz", "compiz-dev"])
        p.manualDependencies = ["manual_dep-1", "manual_dep-2"]
        pd.generateDeps(p)
        expected = set(self.generic_repo["compiz"]["build_dependencies"])
        expected = expected.union(set(["manual_dep-1", "manual_dep-2"]))
        self.assertEqual(expected, p.build_dependencies)

    def test_GenerateDeps_with_no_build_dependencies(self):
        """Verify that no dependencies are found for the project"""
        fake_repo = PackageRepositoryFake()
        #fake_repo.build_depends = set(self.generic_build_depends)
        fake_repo.repo = self.generic_repo
        graph = networkx.DiGraph()
        pd = PackageDependencies(fake_repo, graph)
        p = project.Project("libunity", -1)
        p.packages = set(["libunity9", "libunity-dev"])
        p.manualDependencies = []
        pd.generateDeps(p)
        expected = set()
        self.assertEqual(expected, p.build_dependencies)

    def test_add_to_dependency_graph_single(self):
        """Verify that a single node is added to the graph for project"""
        fake_repo = PackageRepositoryFake()
        #fake_repo.build_depends = set(self.generic_build_depends)
        fake_repo.repo = self.generic_repo
        graph = networkx.DiGraph()
        pd = PackageDependencies(fake_repo, graph)
        p = project.Project("unity", -1)
        p.packages = set(self.generic_repo["unity"]["binaries"])
        p.manualDependencies = []
        pd.add_to_dependency_graph(p)
        self.assertTrue(graph.has_node(p))
        self.assertEqual(graph.number_of_nodes(), 1)

    def test_add_to_dependency_graph_find_build_dependency(self):
        """Verify that a binary package fulfills a build dependency"""
        fake_repo = PackageRepositoryFake()
        #fake_repo.build_depends = set(self.generic_build_depends)
        fake_repo.repo = self.generic_repo
        graph = networkx.DiGraph()
        pd = PackageDependencies(fake_repo, graph)
        unity = project.Project("unity", -1)
        unity.build_dependencies = (
                set(self.generic_repo["unity"]["build_dependencies"]))
        unity.manualDependencies = []
        compiz = project.Project("compiz", -1)
        compiz.packages = set(self.generic_repo["compiz"]["binaries"])
        compiz.manualDependencies = []
        pd.add_to_dependency_graph(unity)
        pd.add_to_dependency_graph(compiz)
        self.assertTrue(graph.has_node(unity))
        self.assertTrue(graph.has_node(compiz))
        self.assertEqual(graph.number_of_nodes(), 2)
        self.assertTrue(graph.has_edge(compiz, unity))
        self.assertFalse(graph.has_edge(unity, compiz))

    def test_add_to_dependency_graph_find_binary_package(self):
        """Verify that a build dependency is fulfilled by a binary package"""
        fake_repo = PackageRepositoryFake()
        #fake_repo.build_depends = set(self.generic_build_depends)
        fake_repo.repo = self.generic_repo
        graph = networkx.DiGraph()
        pd = PackageDependencies(fake_repo, graph)
        unity = project.Project("unity", -1)
        unity.build_dependencies = (
                set(self.generic_repo["unity"]["build_dependencies"]))
        unity.manualDependencies = []
        compiz = project.Project("compiz", -1)
        compiz.packages = set(self.generic_repo["compiz"]["binaries"])
        compiz.manualDependencies = []
        pd.add_to_dependency_graph(compiz)
        pd.add_to_dependency_graph(unity)
        self.assertTrue(graph.has_node(unity))
        self.assertTrue(graph.has_node(compiz))
        self.assertEqual(graph.number_of_nodes(), 2)
        self.assertTrue(graph.has_edge(compiz, unity))
        self.assertFalse(graph.has_edge(unity, compiz))

    def test_add_to_dependency_graph_unrelated(self):
        """Verify that two unrelated packages are not connected"""
        fake_repo = PackageRepositoryFake()
        fake_repo.repo = self.generic_repo
        graph = networkx.DiGraph()
        pd = PackageDependencies(fake_repo, graph)
        nux = project.Project("nux", -1)
        nux.build_dependencies = (
                set(self.generic_repo["nux"]["build_dependencies"]))
        nux.manualDependencies = []
        compiz = project.Project("compiz", -1)
        compiz.packages = set(self.generic_repo["compiz"]["binaries"])
        compiz.manualDependencies = []
        pd.add_to_dependency_graph(compiz)
        pd.add_to_dependency_graph(nux)
        self.assertTrue(graph.has_node(nux))
        self.assertTrue(graph.has_node(compiz))
        self.assertEqual(graph.number_of_nodes(), 2)
        self.assertFalse(graph.has_edge(compiz, nux))
        self.assertFalse(graph.has_edge(nux, compiz))

    def test_record_project_dependencies(self):
        """Verify that the upstream and downstream values are correct when
        using two connected projects"""
        fake_repo = PackageRepositoryFake()
        fake_repo.repo = self.generic_repo
        graph = networkx.DiGraph()
        pd = PackageDependencies(fake_repo, graph)
        unity = project.Project("unity", -1)
        unity.build_dependencies = (
                set(self.generic_repo["unity"]["build_dependencies"]))
        unity.manualDependencies = []
        unity.parameters["target_branch"] = "lp:unity"
        compiz = project.Project("compiz", -1)
        compiz.packages = set(self.generic_repo["compiz"]["binaries"])
        compiz.manualDependencies = []
        compiz.parameters["target_branch"] = "lp:compiz"
        pd.add_to_dependency_graph(unity)
        pd.add_to_dependency_graph(compiz)
        pd.recordProjectDependencies(unity)
        pd.recordProjectDependencies(compiz)
        self.assertEqual(unity.upStream, ["lp:compiz"])
        self.assertEqual(unity.downStream, [])
        self.assertEqual(compiz.upStream, [])
        self.assertEqual(compiz.downStream, ["unity"])

    def test_record_project_dependencies_with_no_target(self):
        """Verify that all upstream and downstream values are [] when no
        target_branch is specified"""
        fake_repo = PackageRepositoryFake()
        fake_repo.repo = self.generic_repo
        graph = networkx.DiGraph()
        pd = PackageDependencies(fake_repo, graph)
        unity = project.Project("unity", -1)
        unity.build_dependencies = (
                set(self.generic_repo["unity"]["build_dependencies"]))
        unity.manualDependencies = []
        unity.parameters["target_branch"] = None
        compiz = project.Project("compiz", -1)
        compiz.packages = set(self.generic_repo["compiz"]["binaries"])
        compiz.manualDependencies = []
        compiz.parameters["target_branch"] = None
        pd.add_to_dependency_graph(unity)
        pd.add_to_dependency_graph(compiz)
        pd.recordProjectDependencies(unity)
        pd.recordProjectDependencies(compiz)
        self.assertEqual(unity.upStream, [])
        self.assertEqual(unity.downStream, [])
        self.assertEqual(compiz.upStream, [])
        self.assertEqual(compiz.downStream, [])
