import unittest
import os.path
import project
from ProjectConfig import *
from PackageRepositoryFake import PackageRepositoryFake

DATA_PATH = "tests/data"
PROJECT_CONFIG_NAME = "project_tests.cfg"


class ProjectConfigTestCase(unittest.TestCase):
    """Unit tests for class ProjectConfig"""

    def setUp(self):
        """Setup for all test cases"""
        self.project_config = ProjectConfig(os.path.join(DATA_PATH,
                                                         PROJECT_CONFIG_NAME))

    def tearDown(self):
        """Tear down for all test cases"""
        pass

    def testGetRoot(self):
        """Verifies getRoot returns the root project name in the config"""
        self.assertEqual(self.project_config.getRoot(), "unity")

    def testRootSection(self):
        """Verifies the root and series attributes match the config"""
        self.assertEqual(self.project_config.root, "unity")
        self.assertEqual(self.project_config.series, "quantal")

    def testGlobalSection(self):
        """Verifies the global entries appear in each config section"""
        global_section = {"pbuilder_hooks": "A00hook B01hook C02hook C00hook",
                          "local_archive": "archive_name",
                          "local_archive_host": "server_host",
                          "local_archive_login": "server_login",
                          "local_archive_tmp": "/tmp/archive",
                          "local_archive_source": "http://test.server/",
                          "local_archive_pocket": "quantal",
                          "ppa": "ppa:test/ppa"}
        table = self.project_config.table
        for section in table:
            for key in global_section:
                self.assertEqual(table[section][key], global_section[key])

    def testStr(self):
        """Verifies the string representation is sane and contains all
        sections"""
        self.assertIn("[root: 'unity']", self.project_config.__str__())
        self.assertIn(" -> ", self.project_config.__str__())
        self.assertIn("'bamf'", self.project_config.__str__())
        self.assertIn("'compiz'", self.project_config.__str__())
        self.assertIn("'dee'", self.project_config.__str__())
        self.assertIn("'nux'", self.project_config.__str__())

    def testGetBinariesFromSource(self):
        """Verifies getBinariesFromSource returns the apt packages when they
        are present"""
        source = "source-one"
        provides = ["binary-one", "binary-two", "binary-three"]
        apt_sources = ["apt-one", "apt-two", "apt-three"]
        repo = PackageRepositoryFake()
        repo.binaries = apt_sources
        ret = self.project_config.getBinariesFromSource(source, provides,
                                                        repo)
        self.assertEqual(ret, set(apt_sources))

    def testGetBinariesFromSourceNone(self):
        """Verifies getBinariesFromSource returns the supplied packages when
        the apt packages are missing"""
        source = "source-one"
        provides = ["binary-one", "binary-two", "binary-three"]
        apt_sources = ["apt-one", "apt-two", "apt-three"]
        repo = PackageRepositoryFake()
        ret = self.project_config.getBinariesFromSource(source, provides,
                                                        repo)
        self.assertEqual(ret, provides)

    def testReviseProjectInfo(self):
        """Verifies reviseProjectInfo returns True for a known project"""
        p = project.Project("compiz", 1)
        ret = self.project_config.reviseProjectInfo(p)
        self.assertEqual(ret, True)

    def testReviseProjectInfoNotFound(self):
        """Verifies reviseProjectInfo returns False for an unknown project"""
        p = project.Project("foo", 1)
        ret = self.project_config.reviseProjectInfo(p)
        self.assertEqual(ret, False)

    def testReviseProjectInfoDetails(self):
        """Verifies reviseProjectInfo updates the project attributes"""
        p = project.Project("bamf", 1)
        self.assertEqual(p.parameters["target_branch"], None)
        ret = self.project_config.reviseProjectInfo(p)
        self.assertEqual(p.name, "bamf")
        self.assertEqual(p.parameters["target_branch"], "lp:bamf")

    def testReviseProjectInfoRequiresAbsent(self):
        """Verifies reviseProjectInfo supplies an empty manualDependencies
        attribute when a project has none"""
        p = project.Project("dee", 1)
        ret = self.project_config.reviseProjectInfo(p)
        self.assertEqual(p.name, "dee")
        self.assertEqual(p.manualDependencies, [])

    def testReviseProjectInfoRequires(self):
        """Verifies reviseProjectInfo updates the project manualDependencies
        attribute when a project has them"""
        p = project.Project("unity", 1)
        ret = self.project_config.reviseProjectInfo(p)
        self.assertEqual(p.name, "unity")
        self.assertEqual(p.manualDependencies, ['libzeitgeist-dev'])


class ProjectConfigInitializeSourceTestCase(unittest.TestCase):
    """Unit tests for ProjectConfig.initializeSources"""

    def testInitializeSources(self):
        """Verifies initializeSources applies package sources to each
        project"""
        project_config = ProjectConfig(os.path.join(DATA_PATH,
                                                    PROJECT_CONFIG_NAME))
        table = project_config.table
        apt_sources = ["apt-one", "apt-two", "apt-three"]

        repo = PackageRepositoryFake()
        repo.binaries = apt_sources
        project_config.initializeSources(repo)
        for section in table:
            self.assertEqual(table[section]['provides'], set(apt_sources))

    def testInitializeSourcesMissingProvides(self):
        """Verifies initializeSources applies package sources to a project
        with no defined provides"""
        config_file = "project_tests_missing_provides.cfg"
        project_config = ProjectConfig(os.path.join(DATA_PATH, config_file))
        apt_sources = ["apt-one", "apt-two", "apt-three"]
        repo = PackageRepositoryFake()
        repo.binaries = apt_sources
        project_config.initializeSources(repo)
        section = project_config.table['nux']
        self.assertEqual(section['provides'], set(apt_sources))

    def testInitializeSourcesProvides(self):
        """Verifies initializeSources supplies provided package sources to a
        project"""
        config_file = "project_tests.cfg"
        project_config = ProjectConfig(os.path.join(DATA_PATH, config_file))
        repo = PackageRepositoryFake()
        project_config.initializeSources(repo)
        section = project_config.table['unity']
        self.assertEqual(section['provides'], ['unity'])

    def testInitializeSourcesMissingProvidesAndBinaries(self):
        """Verifies initializeSources raises exception when no binaries are
        found for a project"""
        config_file = "project_tests_missing_provides.cfg"
        project_config = ProjectConfig(os.path.join(DATA_PATH, config_file))
        apt_sources = []
        repo = PackageRepositoryFake()
        repo.binaries = apt_sources
        with self.assertRaises(KeyError) as err:
            project_config.initializeSources(repo)
        self.assertIn("No binary packages found for", "%s" % (err.exception))


class ProjectErrorsTestCase(unittest.TestCase):
    """Unit tests for ProjectConfig exceptions"""

    def testMissingRootSection(self):
        """Verifies KeyError is raised when missing the root section"""
        config_file = "project_tests_missing_root_section.cfg"
        with self.assertRaises(KeyError):
            ProjectConfig(os.path.join(DATA_PATH,
                                       config_file))

    def testMissingRootOption(self):
        """Verifies KeyError is raised when missing a required root option"""
        config_file = "project_tests_missing_root_option.cfg"
        with self.assertRaises(KeyError):
            ProjectConfig(os.path.join(DATA_PATH,
                                       config_file))

    def testMissingGlobalSection(self):
        """Verifies KeyError is raised when missing the global section"""
        config_file = "project_tests_missing_global_section.cfg"
        with self.assertRaises(KeyError):
            ProjectConfig(os.path.join(DATA_PATH,
                                       config_file))

    def testMissingGlobalOption(self):
        """Verifies KeyError is raised when missing a required global option"""
        config_file = "project_tests_missing_global_option.cfg"
        with self.assertRaises(KeyError):
            ProjectConfig(os.path.join(DATA_PATH,
                                       config_file))

    def testMissingRequiredOption(self):
        """Verifies KeyError is raised when missing a required project
        option"""
        config_file = "project_tests_missing_section_option.cfg"
        with self.assertRaises(KeyError):
            ProjectConfig(os.path.join(DATA_PATH,
                                       config_file))
